#!/usr/bin/php
<?php

/*
 * MultiSSH
 *
 * Developed by Jawish Hameed
 * Copyright (c) 2008 http://www.jawish.org/
 * Licensed under terms of MIT License.
 * All rights reserved.
 *
 * Changelog:
 * 2008-05-31 - created
 *
 */

// Quit if ssh2 is missing
if (!function_exists('ssh2_connect')) die("The SSH2 extension for PHP is not loaded. Quitting...\r\n");

// Check for help request
if ($argv[1] == '-h' || $argv[1] == '') {
	// Print usage help message
	fwrite(STDOUT, "\r\nMultiSSH\r\n--------\r\nUsage: multissh <iplistfile> <defaultuser> <defaultpass>\r\n");
	
	exit();
}

// Read the ip list and check for failure
if (($iplist = file($argv[1], FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES)) === false) {
	
	// Display error if IP file is missing
	if (!file_exists($argv[1])) {
		fwrite(STDOUT, ">IP list file (". $argv[1] .") is missing.\r\n");
	}
	
	// Display IP file read failure error
	fwrite(STDOUT, ">Error reading IP list file.\r\n");
	
	exit();
}

// Get default username and password if given
$username = $argv[2];
$password = $argv[3];

// Prepare connections array
$conns = array('streams' => array());

// Connect to servers
connect($iplist, $username, $password);

do {
	// Check for data waiting on any of the terminal streams
	if (stream_select($activeconns = array(STDIN), $w = NULL, $e = NULL, 0) < 1) {
		
		// Process each terminal stream with data waiting
		foreach ($conns['streams'] as $index => $stdio) {
			
			$ip = $conns['ips'][$index];
			
			// Check stream status
			if (!feof($stdio)) {
				// Stream active:
				
				// Get data from terminal
				$data = receive($index);
				if (!empty($data)) {
					// Format lines
					$data = str_replace("\r\n", "\r\n>$ip\t", $data);
					
					// Write to screen
					fwrite(STDOUT, ">$ip\t$data\r\n");
				}
			} else {
				// Stream closed:
				
				fwrite(STDOUT, ">$ip\tConnection dropped. Do you want to continue or quit? (c/q): ");
				
				if (trim(input(true)) == 'q') die("Quitting...\r\n");
				
				// Destroy terminal
				unset($conns['streams'][$index]);
			}
			
			// Check if there are any active terminals
			if (count($conns['streams']) == 0) die("No active connections. Quitting...\r\n");
		}
		
	} else {
		
		// Fetch data from input
		$input = input();
		if (!empty($input)) {
			switch (strtolower(trim($input))) {
				case 'ctrl-c':
					$input = chr(3);
					break;
				
				case 'ctrl-d':
					$input = chr(4);
					break;
				
				case 'ctrl-v':
					$input = chr(22);
					break;
				
				case 'ctrl-x':
					$input = chr(24);
					break;
				
				case 'ctrl-z':
					$input = chr(26);
					break;
			}
			
			send($input);
		}
		
	}
		
} while (true);

			
function connect($iplist, $username, $password) {
	global $conns;
	
	// Connect to all given IPs
	fwrite(STDOUT, ">Connecting to all servers...\r\n");
	foreach ($iplist as $ipport) {
		
		// Get connection details
		list($ipport, $user, $pass) = explode("\t", trim($ipport));
		list($ip, $port) = explode(':', trim($ipport));
		$username = (!empty($user)) ? $user : $username;
		$password = (!empty($pass)) ? $pass : $password;
		
		// Attempt connection
		$conn = ssh2_connect($ip, $port);
		if ($conn) {
			// Connect successful:
			
			if (ssh2_auth_password($conn, $username, $password)) {
				// Authentication successful:
				
				// Get an available id
				$index = 0;
				while (array_key_exists($index, $conns['streams'])) {
					$index++;
				}
				
				// Initialize data receive cache
				$recv_cache[$index] = NULL;
				
				// Try get an interactive shell
				$stdio = ssh2_shell($conn, 'xterm');
				if ($stdio) {
					stream_set_blocking($stdio, false);
					
					// Add to connections list using id
					$conns['streams'][$index] = $stdio;
					$conns['ips'][$index] = $ip;
					
					fwrite(STDOUT, ">$ip\tConnected successfully\r\n");
				} else {
					fwrite(STDOUT, ">$ip\tInteractive shell failed. Do you want to continue or quit? (c/q): ");
					
					if (trim(input(true)) == 'q') die("Quitting...\r\n");
				}
			
			} else {
				// Authentication failed:
				fwrite(STDOUT, ">$ip\tAuthentication failed. Do you want to continue or quit? (c/q): ");
				
				if (trim(input(true)) == 'q') die("Quitting...\r\n");
			}
		} else {
			// Connect failed:
			
			fwrite(STDOUT, ">$ip\tConnect failed. Do you want to continue or quit? (c/q): ");
			
			if (trim(input(true)) == 'q') die("Quitting...\r\n");
		}
	}
}


function input($wait = false)
{
	$input = '';
	
	do {
		// Get data from input
		$input = fgets(STDIN);
		
		// Wait for data to come about?
		if ($wait == false) break;
	} while (empty($input));
	
	// Return input data
	return $input;
}


function send($data, $index = NULL)
{
	global $conns;
	
	if (!empty($index)) {
		// Send data to terminal at index
		fwrite($conns['streams'][$index], $data);
		
	} else {
		// Send data to all terminals
		foreach ($conns['streams'] as $index => $conn) {
			fwrite($conn, $data);
		}
	}
}


function receive($index)
{
	global $conns;
	
	// Fetch all data from terminal
	$data = '';
	do {
		$line = fgets($conns['streams'][$index]);
		$data .= $line;
	} while (!empty($line));
	
	return $data;
}

?>